<?php
// diviText is a graphical text segmentation tool for use in text mining.
//     Copyright (C) 2011 Amos Jones and Lexomics Research Group
// 
//     This program is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.

require_once( $MODTEXTS );
//require_once( $MODCOUNTER );
require_once( $MODCUTTER );
require_once( $MODDIR );


class Chunkset
{
    // public
    
    // constructor
    public function __construct()
    {

    }

    // simple GETters
    public function GET_name()   { return $this->name; }
    public function GET_folder() { return $this->folder; }
    public function GET_id()     { return $this->id; }
    public function GET_spaces() { return $this->spaces; }

    // better getters

    // simple setters
    public function SET_name( $_n )   { $this->name   = $_n; }
    public function SET_folder( $_f ) { $this->folder = $_f; }
    public function SET_id( $_o )     { $this->id     = $_o; }

    // chunkers
    public function chunk( $text, $spaces )
    {
        $this->spaces = $spaces;
        $cwd = getcwd();
        mkdir( $this->folder, 0700 );
        chdir( $this->folder );
        
        $folders = array( "clean/txt", "clean/csv", "orig/txt", "orig/csv" );

        foreach ( $folders as $f )
        {
            if ( !mkdir( "./$f", 0700, true ) )
                $errors[] = "Could not make folder '$f'.";
        }

        $out = $this->chunker( $text, $spaces, "orig" );
        $out2 = $this->chunker( $text, $spaces, "clean" );

        chdir( $cwd );
        if ( $out || $out2 )
            return true;//array_merge( $out, $out2 );
        //elseif ( $out )
        //    return $out;
        else
            return null;
    }

    private function chunker( $text, $spaces, $style )
    {
        $errors = null;
        $text = collapse_spaces( $text );
        $textarr = split_string( $text );
        $chunksarr = split_on_spaces( $textarr, $spaces );

        if ( $style == "clean" )
            $chunksarr = remove_junk( $chunksarr );

        $chunkhashes = null;
        foreach( $chunksarr as $end => $chunkarr )
            $chunkhashes[$end] = count_words( $chunkarr );

        if ( !$chunkhashes )
        {
            $errors[] = "Could not hash chunks. Huh.";
            trigger_error( "Could not hash chunks. Huh." );
            return $errors;
        }

        // if cleaned style, remove all bad words and lc

        $max = array_pop( array_keys( $textarr ) ) + 1;
        $maxlen = strlen( "$max" );
        $pad = "%0{$maxlen}s";
        foreach ( $chunksarr as $end => $chunkarr )
        {
            $endpad = $end + 1;
            $endpad = sprintf( $pad, $endpad );

            $out  = $this->write_txt( $chunkarr, $endpad, $style );
            $out2 = $this->write_csv( $chunkhashes[$end], $endpad, $style );

            if ( $out || $out2 )
                $errors = array_merge( $out, $out2, $errors );
        }

        if ( $errors )
        {
            rrmdir( $this->folder );
            trigger_error( "Something in the chunking process went wrong." );
        }

        return $errors;

    }

    private function write_txt( $text, $end, $style )
    {
        $file = $this->folder . "/$style/txt/" . $this->id . "_$end.txt";
        $FH = fopen( $file, 'w' );

        if ( !$FH )
            return "Could not open file '$file' in mode 'c+'.";

        if ( !fwrite( $FH, implode( ' ', $text ) ) )
            return "Write to '$file' failed.";

        fclose( $FH );

        return null;
    }

    private function write_csv( $hash, $end, $style )
    {
        $file = $this->folder . "/$style/csv/" . $this->id . "_$end.csv";
        $FH = fopen( $file, 'w' );

        if ( !$FH )
            return "Could not open file '$file' in mode 'c+'.";

        $total = get_total( $hash );
        $unique = get_unique( $hash );
        $hapax = get_hapax( $hash );

        hash_sort( $hash, 'c' );

        // start the csv
        $csv = $this->id . "txt,$total,$unique,$hapax\n";
        $csv .= "RANK,WORD,COUNT,RELATIVE FREQUENCY\n";
        $rank = 0;
        $allrank = 1;
        $prevcount = null;

        // iterate through the list of word printing them out
        foreach ( $hash as $word => $count ) {

            $prop = round( ($count / $total), 10 );

            if ( $count != $prevcount ) 
                $rank = $allrank;
            $prevcount = $count;
            $allrank++;

            // update csv
            $csv .= "$rank,$word,$count,$prop\n";

        }
        
        if ( !fwrite( $FH, $csv ) )
            return "Write to '$file' failed.";

        fclose( $FH );

        return null;
    }

    // remove_chunkset
    public function remove_chunkset()
    {
        rrmdir( $this->folder );
    }

    // private
    private $name      = "";
    private $folder    = "";
    private $id        = "";
};



?>
